=begin
    yagettext.rb - Yet Another GetText
    Copyright (C) 2001  Masahiro Sakai

        Masahiro Sakai                  <s01397ms@sfc.keio.ac.jp>

    You can redistribute this file and/or modify it under the same term
    of Ruby.  License of Ruby is included with Ruby distribution in
    the file "README".
=end

require 'mo.rb'

module YAGetText
  class TextDomain
    def initialize(name, dirname = '.')
      @name     = name
      @dirname  = dirname
      @mofiles  = Hash.new
    end

    def bind(dirname)
      @dirname = dirname
      @mofiles.clear
    end

    def gettext(msgid, category = 'LC_MESSAGES', locale = get_locale(category))
      mo = get_mo(category, locale)
      (mo and mo[msgid]) or msgid
    end

    def [](msgid)
      gettext(msgid)
    end

    private

    def get_locale(category)
      ENV['LANG'] or ENV['LC_ALL'] or ENV[category]
    end

    def locale_candidates(locale)
      locale = String(locale)
      result = []

      matched = /^([A-Za-z]+)(_[A-Za-z]+)?(\.\w+)?((?:@[^@]*)*)/.match(locale)
      if matched.size > 1
        matched = matched.to_a
        matched.shift
        while not matched.empty?
          result.push(matched.join(''))
          matched.pop
        end
        result.uniq!
      else
        result.push(locale)
      end

      result
    end

    def get_mo(category, locale)
      return nil unless locale

      category = String(category)
      locale   = String(locale)
      key      = [category, locale]

      if @mofiles.has_key? key
        @mofiles[key]
      else
        locale_candidates(locale).each{|locale|
          fname = File.join(@dirname, locale, category, @name + '.mo')
          if File.exist? fname
            mo = MOFile.open(fname)
            @mofiles[key] = mo
            return mo
          end
        }
        nil
      end
    end    
  end

  def bindtextdomain(domain, dirname)
    @__text_domain = TextDomain.new(domain, dirname)
    nil
  end
  module_function :bindtextdomain

  def _(str)
    @__text_domain.gettext(str)
  end
  module_function :_
end

# Test
if $0 == __FILE__
  include YAGetText
  bindtextdomain('libc', '/usr/share/locale')
  puts _("Terminated")
end
